@@ -15,26 +15,25 @@
 """Support for beets plugins."""
 
 
-import traceback
-import re
-import inspect
 import abc
+import inspect
+import re
+import traceback
 from collections import defaultdict
 from functools import wraps
 
+import mediafile
 
 import beets
 from beets import logging
-import mediafile
 
-
-PLUGIN_NAMESPACE = 'beetsplug'
+PLUGIN_NAMESPACE = "beetsplug"
 
 # Plugins using the Last.fm API can share the same API key.
-LASTFM_KEY = '2dc3914abf35f0d9c92d97d8f8e42b43'
+LASTFM_KEY = "2dc3914abf35f0d9c92d97d8f8e42b43"
 
 # Global logger.
-log = logging.getLogger('beets')
+log = logging.getLogger("beets")
 
 
 class PluginConflictException(Exception):
@@ -51,11 +50,10 @@ class PluginLogFilter(logging.Filter):
     """
 
     def __init__(self, plugin):
-        self.prefix = f'{plugin.name}: '
+        self.prefix = f"{plugin.name}: "
 
     def filter(self, record):
-        if hasattr(record.msg, 'msg') and isinstance(record.msg.msg,
-                                                     str):
+        if hasattr(record.msg, "msg") and isinstance(record.msg.msg, str):
             # A _LogMessage from our hacked-up Logging replacement.
             record.msg.msg = self.prefix + record.msg.msg
         elif isinstance(record.msg, str):
@@ -65,6 +63,7 @@ class PluginLogFilter(logging.Filter):
 
 # Managing the plugins themselves.
 
+
 class BeetsPlugin:
     """The base class for all beets plugins. Plugins provide
     functionality by defining a subclass of BeetsPlugin and overriding
@@ -72,9 +71,8 @@ class BeetsPlugin:
     """
 
     def __init__(self, name=None):
-        """Perform one-time plugin setup.
-        """
-        self.name = name or self.__module__.split('.')[-1]
+        """Perform one-time plugin setup."""
+        self.name = name or self.__module__.split(".")[-1]
         self.config = beets.config[self.name]
         if not self.template_funcs:
             self.template_funcs = {}
@@ -97,10 +95,11 @@ class BeetsPlugin:
         return ()
 
     def _set_stage_log_level(self, stages):
-        """Adjust all the stages in `stages` to WARNING logging level.
-        """
-        return [self._set_log_level_and_params(logging.WARNING, stage)
-                for stage in stages]
+        """Adjust all the stages in `stages` to WARNING logging level."""
+        return [
+            self._set_log_level_and_params(logging.WARNING, stage)
+            for stage in stages
+        ]
 
     def get_early_import_stages(self):
         """Return a list of functions that should be called as importer
@@ -134,12 +133,11 @@ class BeetsPlugin:
         def wrapper(*args, **kwargs):
             assert self._log.level == logging.NOTSET
 
-            verbosity = beets.config['verbose'].get(int)
+            verbosity = beets.config["verbose"].get(int)
             log_level = max(logging.DEBUG, base_log_level - 10 * verbosity)
             self._log.setLevel(log_level)
             if argspec.varkw is None:
-                kwargs = {k: v for k, v in kwargs.items()
-                          if k in argspec.args}
+                kwargs = {k: v for k, v in kwargs.items() if k in argspec.args}
 
             try:
                 return func(*args, **kwargs)
@@ -149,8 +147,7 @@ class BeetsPlugin:
         return wrapper
 
     def queries(self):
-        """Should return a dict mapping prefixes to Query subclasses.
-        """
+        """Should return a dict mapping prefixes to Query subclasses."""
         return {}
 
     def track_distance(self, item, info):
@@ -201,6 +198,7 @@ class BeetsPlugin:
         """
         # Defer import to prevent circular dependency
         from beets import library
+
         mediafile.MediaFile.add_field(name, descriptor)
         library.Item._media_fields.add(name)
 
@@ -208,8 +206,7 @@ class BeetsPlugin:
     listeners = None
 
     def register_listener(self, event, func):
-        """Add a function as a listener for the specified event.
-        """
+        """Add a function as a listener for the specified event."""
         wrapped_func = self._set_log_level_and_params(logging.WARNING, func)
 
         cls = self.__class__
@@ -230,11 +227,13 @@ class BeetsPlugin:
         function will be invoked as ``%name{}`` from path format
         strings.
         """
+
         def helper(func):
             if cls.template_funcs is None:
                 cls.template_funcs = {}
             cls.template_funcs[name] = func
             return func
+
         return helper
 
     @classmethod
@@ -244,11 +243,13 @@ class BeetsPlugin:
         strings. The function must accept a single parameter, the Item
         being formatted.
         """
+
         def helper(func):
             if cls.template_fields is None:
                 cls.template_fields = {}
             cls.template_fields[name] = func
             return func
+
         return helper
 
 
@@ -262,25 +263,29 @@ def load_plugins(names=()):
     BeetsPlugin subclasses desired.
     """
     for name in names:
-        modname = f'{PLUGIN_NAMESPACE}.{name}'
+        modname = f"{PLUGIN_NAMESPACE}.{name}"
         try:
             try:
                 namespace = __import__(modname, None, None)
             except ImportError as exc:
                 # Again, this is hacky:
-                if exc.args[0].endswith(' ' + name):
-                    log.warning('** plugin {0} not found', name)
+                if exc.args[0].endswith(" " + name):
+                    log.warning("** plugin {0} not found", name)
                 else:
                     raise
             else:
                 for obj in getattr(namespace, name).__dict__.values():
-                    if isinstance(obj, type) and issubclass(obj, BeetsPlugin) \
-                            and obj != BeetsPlugin and obj not in _classes:
+                    if (
+                        isinstance(obj, type)
+                        and issubclass(obj, BeetsPlugin)
+                        and obj != BeetsPlugin
+                        and obj not in _classes
+                    ):
                         _classes.add(obj)
 
         except Exception:
             log.warning(
-                '** error loading plugin {}:\n{}',
+                "** error loading plugin {}:\n{}",
                 name,
                 traceback.format_exc(),
             )
@@ -311,9 +316,9 @@ def find_plugins():
 
 # Communication with plugins.
 
+
 def commands():
-    """Returns a list of Subcommand objects from all loaded plugins.
-    """
+    """Returns a list of Subcommand objects from all loaded plugins."""
     out = []
     for plugin in find_plugins():
         out += plugin.commands()
@@ -332,16 +337,16 @@ def queries():
 
 def types(model_cls):
     # Gives us `item_types` and `album_types`
-    attr_name = f'{model_cls.__name__.lower()}_types'
+    attr_name = f"{model_cls.__name__.lower()}_types"
     types = {}
     for plugin in find_plugins():
         plugin_types = getattr(plugin, attr_name, {})
         for field in plugin_types:
             if field in types and plugin_types[field] != types[field]:
                 raise PluginConflictException(
-                    'Plugin {} defines flexible field {} '
-                    'which has already been defined with '
-                    'another type.'.format(plugin.name, field)
+                    "Plugin {} defines flexible field {} "
+                    "which has already been defined with "
+                    "another type.".format(plugin.name, field)
                 )
         types.update(plugin_types)
     return types
@@ -349,7 +354,7 @@ def types(model_cls):
 
 def named_queries(model_cls):
     # Gather `item_queries` and `album_queries` from the plugins.
-    attr_name = f'{model_cls.__name__.lower()}_queries'
+    attr_name = f"{model_cls.__name__.lower()}_queries"
     queries = {}
     for plugin in find_plugins():
         plugin_queries = getattr(plugin, attr_name, {})
@@ -362,6 +367,7 @@ def track_distance(item, info):
     Returns a Distance object.
     """
     from beets.autotag.hooks import Distance
+
     dist = Distance()
     for plugin in find_plugins():
         dist.update(plugin.track_distance(item, info))
@@ -371,6 +377,7 @@ def track_distance(item, info):
 def album_distance(items, album_info, mapping):
     """Returns the album distance calculated by plugins."""
     from beets.autotag.hooks import Distance
+
     dist = Distance()
     for plugin in find_plugins():
         dist.update(plugin.album_distance(items, album_info, mapping))
@@ -378,23 +385,21 @@ def album_distance(items, album_info, mapping):
 
 
 def candidates(items, artist, album, va_likely, extra_tags=None):
-    """Gets MusicBrainz candidates for an album from each plugin.
-    """
+    """Gets MusicBrainz candidates for an album from each plugin."""
     for plugin in find_plugins():
-        yield from plugin.candidates(items, artist, album, va_likely,
-                                     extra_tags)
+        yield from plugin.candidates(
+            items, artist, album, va_likely, extra_tags
+        )
 
 
 def item_candidates(item, artist, title):
-    """Gets MusicBrainz candidates for an item from the plugins.
-    """
+    """Gets MusicBrainz candidates for an item from the plugins."""
     for plugin in find_plugins():
         yield from plugin.item_candidates(item, artist, title)
 
 
 def album_for_id(album_id):
-    """Get AlbumInfo objects for a given ID string.
-    """
+    """Get AlbumInfo objects for a given ID string."""
     for plugin in find_plugins():
         album = plugin.album_for_id(album_id)
         if album:
@@ -402,8 +407,7 @@ def album_for_id(album_id):
 
 
 def track_for_id(track_id):
-    """Get TrackInfo objects for a given ID string.
-    """
+    """Get TrackInfo objects for a given ID string."""
     for plugin in find_plugins():
         track = plugin.track_for_id(track_id)
         if track:
@@ -439,29 +443,44 @@ def import_stages():
 
 # New-style (lazy) plugin-provided fields.
 
+
+def _check_conflicts_and_merge(plugin, plugin_funcs, funcs):
+    """Check the provided template functions for conflicts and merge into funcs.
+
+    Raises a `PluginConflictException` if a plugin defines template functions
+    for fields that another plugin has already defined template functions for.
+    """
+    if plugin_funcs:
+        if not plugin_funcs.keys().isdisjoint(funcs.keys()):
+            conflicted_fields = ", ".join(plugin_funcs.keys() & funcs.keys())
+            raise PluginConflictException(
+                f"Plugin {plugin.name} defines template functions for "
+                f"{conflicted_fields} that conflict with another plugin."
+            )
+        funcs.update(plugin_funcs)
+
+
 def item_field_getters():
     """Get a dictionary mapping field names to unary functions that
     compute the field's value.
     """
     funcs = {}
     for plugin in find_plugins():
-        if plugin.template_fields:
-            funcs.update(plugin.template_fields)
+        _check_conflicts_and_merge(plugin, plugin.template_fields, funcs)
     return funcs
 
 
 def album_field_getters():
-    """As above, for album fields.
-    """
+    """As above, for album fields."""
     funcs = {}
     for plugin in find_plugins():
-        if plugin.album_template_fields:
-            funcs.update(plugin.album_template_fields)
+        _check_conflicts_and_merge(plugin, plugin.album_template_fields, funcs)
     return funcs
 
 
 # Event dispatch.
 
+
 def event_handlers():
     """Find all event handlers from plugins as a dictionary mapping
     event names to sequences of callables.
@@ -482,7 +501,7 @@ def send(event, **arguments):
 
     Return a list of non-None values returned from the handlers.
     """
-    log.debug('Sending event: {0}', event)
+    log.debug("Sending event: {0}", event)
     results = []
     for handler in event_handlers()[event]:
         result = handler(**arguments)
@@ -497,11 +516,11 @@ def feat_tokens(for_artist=True):
     The `for_artist` option determines whether the regex should be
     suitable for matching artist fields (the default) or title fields.
     """
-    feat_words = ['ft', 'featuring', 'feat', 'feat.', 'ft.']
+    feat_words = ["ft", "featuring", "feat", "feat.", "ft."]
     if for_artist:
-        feat_words += ['with', 'vs', 'and', 'con', '&']
-    return r'(?<=\s)(?:{})(?=\s)'.format(
-        '|'.join(re.escape(x) for x in feat_words)
+        feat_words += ["with", "vs", "and", "con", "&"]
+    return r"(?<=\s)(?:{})(?=\s)".format(
+        "|".join(re.escape(x) for x in feat_words)
     )
 
 
@@ -517,7 +536,7 @@ def sanitize_choices(choices, choices_all):
         if s not in seen:
             if s in list(choices_all):
                 res.append(s)
-            elif s == '*':
+            elif s == "*":
                 res.extend(others)
         seen.add(s)
     return res
@@ -550,11 +569,11 @@ def sanitize_pairs(pairs, pairs_all):
                 if x not in seen:
                     seen.add(x)
                     res.append(x)
-            elif k == '*':
+            elif k == "*":
                 new = [o for o in others if o not in seen]
                 seen.update(new)
                 res.extend(new)
-            elif v == '*':
+            elif v == "*":
                 new = [o for o in others if o not in seen and o[0] == k]
                 seen.update(new)
                 res.extend(new)
@@ -568,12 +587,15 @@ def notify_info_yielded(event):
     Each yielded value is passed to plugins using the 'info' parameter of
     'send'.
     """
+
     def decorator(generator):
         def decorated(*args, **kwargs):
             for v in generator(*args, **kwargs):
                 send(event, info=v)
                 yield v
+
         return decorated
+
     return decorator
 
 
@@ -583,7 +605,7 @@ def get_distance(config, data_source, info):
     """
     dist = beets.autotag.Distance()
     if info.data_source == data_source:
-        dist.add('source', config['source_weight'].as_number())
+        dist.add("source", config["source_weight"].as_number())
     return dist
 
 
@@ -620,7 +642,7 @@ def apply_item_changes(lib, item, move, pretend, write):
 class MetadataSourcePlugin(metaclass=abc.ABCMeta):
     def __init__(self):
         super().__init__()
-        self.config.add({'source_weight': 0.5})
+        self.config.add({"source_weight": 0.5})
 
     @abc.abstractproperty
     def id_regex(self):
@@ -643,7 +665,7 @@ class MetadataSourcePlugin(metaclass=abc.ABCMeta):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def _search_api(self, query_type, filters, keywords=''):
+    def _search_api(self, query_type, filters, keywords=""):
         raise NotImplementedError
 
     @abc.abstractmethod
@@ -655,7 +677,7 @@ class MetadataSourcePlugin(metaclass=abc.ABCMeta):
         raise NotImplementedError
 
     @staticmethod
-    def get_artist(artists, id_key='id', name_key='name'):
+    def get_artist(artists, id_key="id", name_key="name", join_key=None):
         """Returns an artist string (all artists) and an artist_id (the main
         artist) for a list of artist object dicts.
 
@@ -663,6 +685,8 @@ class MetadataSourcePlugin(metaclass=abc.ABCMeta):
         and 'the') to the front and strips trailing disambiguation numbers. It
         returns a tuple containing the comma-separated string of all
         normalized artists and the ``id`` of the main/first artist.
+        Alternatively a keyword can be used to combine artists together into a
+        single string by passing the join_key argument.
 
         :param artists: Iterable of artist dicts or lists returned by API.
         :type artists: list[dict] or list[list]
@@ -673,39 +697,55 @@ class MetadataSourcePlugin(metaclass=abc.ABCMeta):
             to concatenate for the artist string (containing all artists).
             Defaults to 'name'.
         :type name_key: str or int
+        :param join_key: Key or index corresponding to a field containing a
+            keyword to use for combining artists into a single string, for
+            example "Feat.", "Vs.", "And" or similar. The default is None
+            which keeps the default behaviour (comma-separated).
+        :type join_key: str or int
         :return: Normalized artist string.
         :rtype: str
         """
         artist_id = None
-        artist_names = []
-        for artist in artists:
+        artist_string = ""
+        artists = list(artists)  # In case a generator was passed.
+        total = len(artists)
+        for idx, artist in enumerate(artists):
             if not artist_id:
                 artist_id = artist[id_key]
             name = artist[name_key]
             # Strip disambiguation number.
-            name = re.sub(r' \(\d+\)$', '', name)
+            name = re.sub(r" \(\d+\)$", "", name)
             # Move articles to the front.
-            name = re.sub(r'^(.*?), (a|an|the)$', r'\2 \1', name, flags=re.I)
-            artist_names.append(name)
-        artist = ', '.join(artist_names).replace(' ,', ',') or None
-        return artist, artist_id
+            name = re.sub(r"^(.*?), (a|an|the)$", r"\2 \1", name, flags=re.I)
+            # Use a join keyword if requested and available.
+            if idx < (total - 1):  # Skip joining on last.
+                if join_key and artist.get(join_key, None):
+                    name += f" {artist[join_key]} "
+                else:
+                    name += ", "
+            artist_string += name
 
-    def _get_id(self, url_type, id_):
+        return artist_string, artist_id
+
+    @staticmethod
+    def _get_id(url_type, id_, id_regex):
         """Parse an ID from its URL if necessary.
 
         :param url_type: Type of URL. Either 'album' or 'track'.
         :type url_type: str
         :param id_: Album/track ID or URL.
         :type id_: str
+        :param id_regex: A dictionary containing a regular expression
+            extracting an ID from an URL (if it's not an ID already) in
+            'pattern' and the number of the match group in 'match_group'.
+        :type id_regex: dict
         :return: Album/track ID.
         :rtype: str
         """
-        self._log.debug(
-            "Searching {} for {} '{}'", self.data_source, url_type, id_
-        )
-        match = re.search(self.id_regex['pattern'].format(url_type), str(id_))
+        log.debug("Extracting {} ID from '{}'", url_type, id_)
+        match = re.search(id_regex["pattern"].format(url_type), str(id_))
         if match:
-            id_ = match.group(self.id_regex['match_group'])
+            id_ = match.group(id_regex["match_group"])
             if id_:
                 return id_
         return None
@@ -726,11 +766,11 @@ class MetadataSourcePlugin(metaclass=abc.ABCMeta):
         :return: Candidate AlbumInfo objects.
         :rtype: list[beets.autotag.hooks.AlbumInfo]
         """
-        query_filters = {'album': album}
+        query_filters = {"album": album}
         if not va_likely:
-            query_filters['artist'] = artist
-        results = self._search_api(query_type='album', filters=query_filters)
-        albums = [self.album_for_id(album_id=r['id']) for r in results]
+            query_filters["artist"] = artist
+        results = self._search_api(query_type="album", filters=query_filters)
+        albums = [self.album_for_id(album_id=r["id"]) for r in results]
         return [a for a in albums if a is not None]
 
     def item_candidates(self, item, artist, title):
@@ -747,7 +787,7 @@ class MetadataSourcePlugin(metaclass=abc.ABCMeta):
         :rtype: list[beets.autotag.hooks.TrackInfo]
         """
         tracks = self._search_api(
-            query_type='track', keywords=title, filters={'artist': artist}
+            query_type="track", keywords=title, filters={"artist": artist}
         )
         return [self.track_for_id(track_data=track) for track in tracks]
 
