@@ -16,11 +16,12 @@
 """
 
 
-from beets.plugins import BeetsPlugin
-from beets.dbcore.query import StringFieldQuery
-from beets import config
 import difflib
 
+from beets import config
+from beets.dbcore.query import StringFieldQuery
+from beets.plugins import BeetsPlugin
+
 
 class FuzzyQuery(StringFieldQuery):
     @classmethod
@@ -29,18 +30,20 @@ class FuzzyQuery(StringFieldQuery):
         if pattern.islower():
             val = val.lower()
         query_matcher = difflib.SequenceMatcher(None, pattern, val)
-        threshold = config['fuzzy']['threshold'].as_number()
+        threshold = config["fuzzy"]["threshold"].as_number()
         return query_matcher.quick_ratio() >= threshold
 
 
 class FuzzyPlugin(BeetsPlugin):
     def __init__(self):
         super().__init__()
-        self.config.add({
-            'prefix': '~',
-            'threshold': 0.7,
-        })
+        self.config.add(
+            {
+                "prefix": "~",
+                "threshold": 0.7,
+            }
+        )
 
     def queries(self):
-        prefix = self.config['prefix'].as_str()
+        prefix = self.config["prefix"].as_str()
         return {prefix: FuzzyQuery}
