@@ -13,28 +13,29 @@
 # included in all copies or substantial portions of the Software.
 
 
-import confuse
 from sys import stderr
 
-__version__ = '1.6.0'
-__author__ = 'Adrian Sampson <adrian@radbox.org>'
+import confuse
+
+__version__ = '2.0.0'
+__author__ = "Adrian Sampson <adrian@radbox.org>"
 
 
 class IncludeLazyConfig(confuse.LazyConfig):
     """A version of Confuse's LazyConfig that also merges in data from
     YAML files specified in an `include` setting.
     """
+
     def read(self, user=True, defaults=True):
         super().read(user, defaults)
 
         try:
-            for view in self['include']:
+            for view in self["include"]:
                 self.set_file(view.as_filename())
         except confuse.NotFoundError:
             pass
         except confuse.ConfigReadError as err:
-            stderr.write("configuration `import` failed: {}"
-                         .format(err.reason))
+            stderr.write("configuration `import` failed: {}".format(err.reason))
 
 
-config = IncludeLazyConfig('beets', __name__)
+config = IncludeLazyConfig("beets", __name__)
